#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-
import math
from ost import gui
from ost import gfx
try: 
  from ost import img
  _img_present=True
except ImportError:
  _img_present=False
  pass
from PyQt5 import QtCore, QtWidgets

from .preset_widget import PresetWidget

class AdditionalSettingsWidget(QtWidgets.QStackedWidget):
  def __init__(self, parent=None):
    QtWidgets.QStackedWidget.__init__(self, parent)
    self.map_widget_ = MapLevelWidget(self)
    self.preset_widget_ = PresetWidget(self)
    self.addWidget(self.preset_widget_);
    self.addWidget(self.map_widget_);
    self.setContentsMargins(0,0,0,0)
    self.setMinimumSize(self.preset_widget_.minimumSize())
    
  def Update(self):
    self.setEnabled(True)
    scene_selection = gui.SceneSelection.Instance()
    all_img = True
    all_entity = True
    for i in range(0,scene_selection.GetActiveNodeCount()):
      node = scene_selection.GetActiveNode(i)
      if not (isinstance(node, gfx.Entity) or isinstance(node, gfx.Surface)):
        all_entity = False
      if (not _img_present) or (not isinstance(node, gfx.MapIso)):
        all_img = False
    if all_img:
      self.map_widget_.Update()
      self.setMinimumSize(self.map_widget_.minimumSize())
      self.resize(self.map_widget_.minimumSize())
      self.setMinimumSize(self.map_widget_.minimumSize())
      self.setCurrentWidget(self.map_widget_)
    elif all_entity:
      self.preset_widget_.Update()
      self.setMinimumSize(self.preset_widget_.minimumSize())
      self.resize(self.preset_widget_.minimumSize())
      self.setMinimumSize(self.preset_widget_.minimumSize())
      self.setCurrentWidget(self.preset_widget_)
    else:
      self.setEnabled(False)
    
#Map Level Widget
class MapLevelWidget(QtWidgets.QWidget):
  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Create Ui elements
    self.map_level_label_ = QtWidgets.QLabel("Map Contour Level")
    self.font = self.map_level_label_.font()
    self.font.setBold(True)
    
    self.level_preview_ = LevelPreview()
    
    self.level_spinbox_ = QtWidgets.QDoubleSpinBox()
    self.level_spinbox_.setDecimals(3)
    self.level_spinbox_.setSingleStep(0.05)
    
    grid = QtWidgets.QGridLayout()
    grid.setContentsMargins(0,5,0,0)
    grid.addWidget(self.level_preview_, 0, 0, 1, 4)
    grid.addWidget(self.map_level_label_, 1, 0, 1, 3)
    grid.addWidget(self.level_spinbox_,1,3,1,1)
    grid.setRowStretch(3, 1)
    self.setLayout(grid)
    
    self.level_preview_.levelUpdated.connect(self.UpdateLevel)
    self.level_preview_.levelModified.connect(self.ModifySpinBox)
    self.level_spinbox_.valueChanged.connect(self.UpdateLevel)

    self.setMinimumSize(250,200)
        
  def Update(self):
    scene_selection = gui.SceneSelection.Instance()
    if(scene_selection.GetActiveNodeCount()==1):
      node = scene_selection.GetActiveNode(0)
      if _img_present and isinstance(node, gfx.MapIso):
        try:
          self.level_preview_.SetBins(node.GetHistogram())
          self.level_preview_.SetMinimum(node.GetMinLevel())
          self.level_spinbox_.setMinimum(node.GetMinLevel())
          self.level_preview_.SetMaximum(node.GetMaxLevel())
          self.level_spinbox_.setMaximum(node.GetMaxLevel())
          self.level_preview_.SetLevel(node.GetLevel())
          self.level_spinbox_.show()
          self.font.setBold(True)
          self.map_level_label_.setText("Map Contour Level")
          self.setEnabled(True)
        except UserWarning:
          self.font.setBold(False)
          self.map_level_label_.setText("Map uniformly filled with level %s"%node.GetLevel())
          self.level_spinbox_.hide()
          self.setEnabled(False)
      else:
        self.setEnabled(False)
    else:
      self.setEnabled(False)
        
  def UpdateLevel(self, level):
    scene_selection = gui.SceneSelection.Instance()
    if(scene_selection.GetActiveNodeCount()==1):
      node = scene_selection.GetActiveNode(0)
      node.SetLevel(level)
      
  def ModifySpinBox(self, level):
    self.level_spinbox_.valueChanged.disconnect(self.UpdateLevel)
    self.level_spinbox_.setValue(level)
    self.level_spinbox_.valueChanged.connect(self.UpdateLevel)
        
#Level Preview
class LevelPreview(QtWidgets.QWidget):

  levelUpdated = QtCore.pyqtSignal(int, name="levelUpdated")
  levelModified = QtCore.pyqtSignal(int, name="levelModified")

  def __init__(self, parent=None):
    QtWidgets.QWidget.__init__(self, parent)
    
    #Defaults
    self.border_offset_ = 3
    self.preview_height_ = 150
    QtWidgets.QWidget.__init__(self, parent)
    
    #Ui
    self.setMinimumSize(0, self.preview_height_ + 4)

    self.bins_ = None
    self.level_ = 0
    self.minimum_ = 0
    self.maximum_ = 0
    
    self.paint_mouse_=False
    
  def SetBins(self, bins):
    self.bins_ = bins
    self.update()

  def SetMaximum(self, max):
    self.maximum_ = max

  def SetMinimum(self, min):
    self.minimum_ = min
    
  def SetLevel(self, level):
    self.level_ = level

  def GetLevel(self):
    return self.level_
    
  def paintEvent(self, event):   
    if self.isEnabled() and self.bins_ is not None:
      painter = QtWidgets.QPainter()
      if painter.begin(self):
        self.PaintBackground(painter)
        self.PaintBins(painter)
        self.PaintLevel(painter)
        if(self.paint_mouse_):
          self.PaintMouse(painter)
      painter.end()

  def PaintBackground(self,painter):
    size = self.size()
    painter.setBrush(QtCore.Qt.white)
    painter.setPen(QtCore.Qt.white)
    painter.drawRect(self.border_offset_,
                   self.border_offset_,
                   size.width() - 2 * self.border_offset_,
                   self.preview_height_)

  def PaintBins(self,painter):
    size = self.size()
    bin_cnt = len(self.bins_)
    bin_width = (size.width()-2* self.border_offset_) / float(bin_cnt)
    max=0
    for b in self.bins_:
      if(b>max):
        max = b
    max = math.log(max)
    if(max > 0):
      painter.setBrush(QtCore.Qt.black)
      painter.setPen(QtCore.Qt.black)
      for i in range(0,bin_cnt):
        bin_height = self.bins_[i]
        if(bin_height>0):
          bin_height = math.floor((math.log(bin_height)/max)*(self.preview_height_-2*(self.border_offset_)))
          painter.drawRect(self.border_offset_ + (i*bin_width),
                         self.preview_height_ - bin_height,
                         bin_width,
                         bin_height)

  def PaintLevel(self,painter):
    size = self.size()
    width = size.width()-(2* self.border_offset_)
    tot_len = self.maximum_-self.minimum_
    if(tot_len>0):
      cur_len = self.level_-self.minimum_
      painter.setBrush(QtCore.Qt.red)
      painter.setPen(QtCore.Qt.red)
      painter.drawRect((width / tot_len) * cur_len,
                     self.border_offset_,
                     1,
                     self.preview_height_)

  def PaintMouse(self,painter):
    size = self.size()
    width = size.width()-(2* self.border_offset_)
    painter.setBrush(QtCore.Qt.gray)
    painter.setPen(QtCore.Qt.gray)
    pos=self.mapFromGlobal(QtWidgets.QCursor.pos())
    painter.drawRect(pos.x(),
                   self.border_offset_,
                   1,
                   self.preview_height_)

  def mouseReleaseEvent(self, event):
    self.paint_mouse_=False
    size = self.size()
    width = size.width()-(2* self.border_offset_)
    tot_len = self.maximum_-self.minimum_
    self.level_ = self.minimum_ + float(event.x())/width * tot_len
    self.update()
    self.levelUpdated.emit(self.level_)
  
  def mousePressEvent(self,event):
    self.paint_mouse_=True
    
  def mouseMoveEvent(self, event):
    size = self.size()
    width = size.width()-(2* self.border_offset_)
    tot_len = self.maximum_-self.minimum_
    level = self.minimum_ + float(event.x())/width * tot_len
    self.levelModified.emit(level)
    self.update()



from ost.mol.mm import  *
from PyQt5 import QtCore
from math import sqrt
from math import sin

"""
MM - Demo

!!!Requires OpenStructure to be compiled with MM-Support!!!

This demo should show the procedure building a topology from scratch and
setting up a simulation. As an additional piece of sugar, it demonstrates
the capabilities of altering force parameters as the simulation runs.
The script can directly be started from within DNG.
"""

class Anim(QtCore.QTimer):
    def __init__(self,sim,go,cc_bond_index):
        QtCore.QTimer.__init__(self)
        self.ent=sim.GetEntity()
        self.sim=sim
        self.go = go
        self.ed = ent.EditXCS()
        self.bond_index = cc_bond_index
        top = sim.GetTopology()
        param = top.GetHarmonicBondParameters(cc_bond_index)
        self.bond_length = param[2]
        self.force_constant = param[3]
        self.steps = 0
        self.counter = 0
        self.timeout.connect(self.OnTimer)

    def OnTimer(self):
        if self.steps % 20 == 0:
          sim.ResetHarmonicBond(self.bond_index,sin(self.counter*3.1416/100)*self.bond_length*0.75 + self.bond_length,self.force_constant)
          self.counter += 1

        self.sim.Steps(1)
        self.sim.UpdatePositions()
        self.go.UpdatePositions()
        self.steps+=1


#create topology by only defining masses

prof = io.IOProfile(dialect='PDB', fault_tolerant=False,
                 quack_mode=False, processor=conop.HeuristicProcessor())

ent = io.LoadPDB('ethanol.pdb', profile=prof)
masses = [12.011,12.011,15.999,1.008,1.008,1.008,1.008,1.008,1.008]
top = Topology(masses)

#all per atom parameters have to be set at once...
charges = [-0.18,0.145,-0.683,0.06,0.06,0.06,0.06,0.06,0.418]
sigmas = [0.37,0.37,0.32,0.24,0.24,0.24,0.24,0.24,0.04]
epsilons = [0.28,0.28,0.64,0.09,0.09,0.09,0.09,0.09,0.2]

top.SetCharges(charges)
top.SetSigmas(sigmas)
top.SetEpsilons(epsilons)

#we can set the fudge parameters for 1,4 interactions also in the topology
top.SetFudgeQQ(0.5)
top.SetFudgeLJ(0.5)

#add the bonds
top.AddHarmonicBond(0,1,0.153,186188.0) #C-C
top.AddHarmonicBond(0,3,0.1111,269449.6) #C-H
top.AddHarmonicBond(0,4,0.1111,269449.6) #C-H
top.AddHarmonicBond(0,5,0.1111,269449.6) #C-H
top.AddHarmonicBond(1,6,0.1111,269449.6) #C-H
top.AddHarmonicBond(1,7,0.1111,269449.6) #C-H
top.AddHarmonicBond(1,2,0.142,358150.4) #C-O
top.AddHarmonicBond(2,8,0.096,456056.0) #O-H

#add the angles
top.AddUreyBradleyAngle(3,0,4,1.892,297.064,0.1802,4518.72) #H-C-H
top.AddUreyBradleyAngle(3,0,5,1.892,297.064,0.1802,4518.72) #H-C-H
top.AddUreyBradleyAngle(4,0,5,1.892,297.064,0.1802,4518.72) #H-C-H
top.AddUreyBradleyAngle(3,0,1,1.922,289.5328,0.2179,18853.104) #H-C-C
top.AddUreyBradleyAngle(4,0,1,1.922,289.5328,0.2179,18853.104) #H-C-C
top.AddUreyBradleyAngle(5,0,1,1.922,289.5328,0.2179,18853.104) #H-C-C
top.AddUreyBradleyAngle(0,1,6,1.922,289.5328,0.2179,18853.104) #H-C-C
top.AddUreyBradleyAngle(0,1,7,1.922,289.5328,0.2179,18853.104) #H-C-C
top.AddUreyBradleyAngle(0,1,2,1.920,633.4576,0.0,0.0) #C-C-O
top.AddUreyBradleyAngle(6,1,2,1.900,384.0912,0.0,0.0) #H-C-O
top.AddUreyBradleyAngle(7,1,2,1.900,384.0912,0.0,0.0) #H-C-O
top.AddUreyBradleyAngle(6,1,7,1.892,297.064,0.1802,4518.72) #H-C-H
top.AddUreyBradleyAngle(1,2,8,1.850,481.16,0.0,0.0) #C-O-H


#add the dihedrals
top.AddPeriodicDihedral(3,0,1,6,3,0.0,0.66944) #H-C-C-H
top.AddPeriodicDihedral(3,0,1,7,3,0.0,0.66944) #H-C-C-H
top.AddPeriodicDihedral(3,0,1,2,3,0.0,0.66944) #H-C-C-O
top.AddPeriodicDihedral(4,0,1,6,3,0.0,0.66944) #H-C-C-H
top.AddPeriodicDihedral(4,0,1,7,3,0.0,0.66944) #H-C-C-H
top.AddPeriodicDihedral(4,0,1,2,3,0.0,0.66944) #H-C-C-O
top.AddPeriodicDihedral(5,0,1,6,3,0.0,0.66944) #H-C-C-H
top.AddPeriodicDihedral(5,0,1,7,3,0.0,0.66944) #H-C-C-H
top.AddPeriodicDihedral(5,0,1,2,3,0.0,0.66944) #H-C-C-O
top.AddPeriodicDihedral(6,1,2,8,3,0.0,0.58576) #H-C-O-H
top.AddPeriodicDihedral(7,1,2,8,3,0.0,0.58576) #H-C-O-H
top.AddPeriodicDihedral(0,1,2,8,1,0.0,5.4392) #C-C-O-H
top.AddPeriodicDihedral(0,1,2,8,2,0.0,1.2552) #C-C-O-H
top.AddPeriodicDihedral(0,1,2,8,3,0.0,1.75728) #C-C-O-H

#we have to manually set the exclusions for the nonbonded interaction...

#1,2 interactions
top.AddExclusion(3,0)
top.AddExclusion(4,0)
top.AddExclusion(5,0)
top.AddExclusion(0,1)
top.AddExclusion(1,6)
top.AddExclusion(1,7)
top.AddExclusion(1,2)
top.AddExclusion(2,8)

#1,3 interactions
top.AddExclusion(3,4)
top.AddExclusion(3,5)
top.AddExclusion(4,5)
top.AddExclusion(3,1)
top.AddExclusion(4,1)
top.AddExclusion(5,1)
top.AddExclusion(0,6)
top.AddExclusion(0,7)
top.AddExclusion(0,2)
top.AddExclusion(6,2)
top.AddExclusion(7,2)
top.AddExclusion(1,8)

#add lj-pairs, note, that the fudge parameters still get applied during system setup
top.AddLJPair(3,6,0.5*(sigmas[3]+sigmas[6]),sqrt(epsilons[3]*epsilons[6]))
top.AddLJPair(3,7,0.5*(sigmas[3]+sigmas[7]),sqrt(epsilons[3]*epsilons[7]))
top.AddLJPair(3,2,0.5*(sigmas[3]+sigmas[2]),sqrt(epsilons[3]*epsilons[2]))
top.AddLJPair(4,6,0.5*(sigmas[4]+sigmas[6]),sqrt(epsilons[4]*epsilons[6]))
top.AddLJPair(4,7,0.5*(sigmas[4]+sigmas[7]),sqrt(epsilons[4]*epsilons[7]))
top.AddLJPair(4,2,0.5*(sigmas[4]+sigmas[2]),sqrt(epsilons[4]*epsilons[2]))
top.AddLJPair(5,6,0.5*(sigmas[5]+sigmas[6]),sqrt(epsilons[5]*epsilons[6]))
top.AddLJPair(5,7,0.5*(sigmas[5]+sigmas[7]),sqrt(epsilons[5]*epsilons[7]))
top.AddLJPair(5,2,0.5*(sigmas[5]+sigmas[2]),sqrt(epsilons[5]*epsilons[2]))
top.AddLJPair(6,8,0.5*(sigmas[6]+sigmas[8]),sqrt(epsilons[6]*epsilons[8]))
top.AddLJPair(7,8,0.5*(sigmas[7]+sigmas[8]),sqrt(epsilons[7]*epsilons[8]))
top.AddLJPair(0,8,0.5*(sigmas[0]+sigmas[8]),sqrt(epsilons[0]*epsilons[8])) 

settings = Settings()
settings.init_temperature =  310
settings.integrator = VerletIntegrator(0.0001)

#let's extract the cc_bond_index to have some fun with it
cc_bond_index = top.GetHarmonicBondIndices(0,1)[0]

#create the simulation
sim = Simulation(top,ent,settings)
go = gfx.Entity("ethanol",sim.GetEntity())
go.SetRenderMode(gfx.CUSTOM)
scene.Add(go)

anim = Anim(sim,go,cc_bond_index)
anim.start(2)





#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------

import sys
from ost import gui
from ost import gfx
import ost
import os
from PyQt5 import QtCore, QtWidgets
from .toolbar_options_widget import ToolBarOptionsWidget 
from .render_options_widget import RenderOptionsWidget
from .color_options_widget import ColorOptionsWidget
from ost.gui.scene.scene_observer_impl import SceneObserverImpl
from .map_level_widget import AdditionalSettingsWidget
from .scene_selection_helper import SelHelper

class InspectorWidget(ToolBarOptionsWidget):
  ICONS_PATH = os.path.join(ost.GetSharedDataPath(), "scene", "icons/")
  def __init__(self, parent=None):
    ToolBarOptionsWidget.__init__(self, parent)
    app=gui.GostyApp.Instance()
    options = [
                [InspectorWidget.ICONS_PATH+"render_icon.png",RenderOptionsWidget(self),None], 
                [InspectorWidget.ICONS_PATH+"color_icon.png",ColorOptionsWidget(self),None],
                [InspectorWidget.ICONS_PATH+"preset_icon.png", AdditionalSettingsWidget(self),"Additional Node Settings"],
                [InspectorWidget.ICONS_PATH+"tool_icon.png",app.tool_options_win.qobject,"Tool Options"]
              ]
    for o in options:
      ToolBarOptionsWidget.AddWidget(self,o[0], o[1], o[2])
    
    self.obs = SceneObserverImpl()
    self.obs.AttachObserver(self)
    ost.scene.AttachObserver(self.obs)
    app.scene_win.qobject.ActiveNodesChanged.connect(self.ActiveNodesChanged)     
    
    self.setMinimumSize(250,215)
  #ToolBarOptionsWidget Method
  def OnComboChange(self, item):
    self.DoResize()
        
  #Observer Methods    
  def NodeRemoved(self, node):
    SelHelper().Update()
    ToolBarOptionsWidget.Update(self)  
  
  def RenderModeChanged(self, node):
    SelHelper().Update()
    ToolBarOptionsWidget.Update(self)
   
  def NodeChanged(self, node):
    SelHelper().Update()
    ToolBarOptionsWidget.Update(self)

  def ActiveNodesChanged(self):
    SelHelper().Update()
    ToolBarOptionsWidget.Update(self)

class InspectorDialog(QtWidgets.QDialog):

  visible = QtCore.pyqtSignal(bool, name="visible")

  def __init__(self, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    self.setWindowTitle("Inspector Gadget")
    self.setAttribute(QtCore.Qt.WA_MacSmallSize)    
    self.layout=QtWidgets.QHBoxLayout()
    self.layout.setContentsMargins(0,0,0,0)
    self.layout.setSpacing(0)
    self.setLayout(self.layout)
    self.mywidget_ = InspectorWidget(self)
    self.layout.addWidget(self.mywidget_)
    size_pol = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding,
                                 QtWidgets.QSizePolicy.Expanding)
    self.setSizePolicy(size_pol)
    self.DoResize()
    
  def DoResize(self):
    if(hasattr(self, "mywidget_")):
      self.setMinimumSize(self.mywidget_.minimumWidth(),
                          self.mywidget_.minimumHeight())
      self.resize(self.mywidget_.minimumWidth(),
                  self.mywidget_.minimumHeight())
    
  def ToggleHide(self,checked):
    self.setHidden(not self.isHidden())

  def hideEvent(self, event):
    self.visible.emit(False)
    QtWidgets.QDialog.hideEvent(self,event)
    
  def showEvent(self, event):
    self.visible.emit(True)
    QtWidgets.QDialog.showEvent(self,event)



import unittest
from ost import *
from ost import settings
from ost.bindings import lga
from ost import testutils

class TestLGABindings(unittest.TestCase):
  
  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb")
    self.chain_a = self.protein.Select("cname=A")

  def testLGA(self):

    try:
      lga_path = settings.Locate('lga')  
    except:
      print("Could not find lga executable: ignoring unit tests")
      return

    lga_result = lga.GDT(self.chain_a, self.chain_a, 
                         reference_length=len(self.chain_a.residues))

    expected_transform = geom.Mat4(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1)
    self.assertEqual(lga_result.gdt_ts, 100.0)
    self.assertEqual(lga_result.gdt_ha, 100.0)
    self.assertEqual(lga_result.GetTransform(), expected_transform)

if __name__ == "__main__":
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
from ._ost_conop import *

# The 20 standard amino acids in no particular order
STANDARD_AMINOACIDS=(
 'ALA', 'ARG', 'ASN',
 'ASP', 'GLN', 'GLU',
 'LYS', 'SER', 'CYS',
 'MET', 'TRP', 'TYR',
 'THR', 'VAL', 'ILE',
 'LEU', 'GLY', 'PRO',
 'HIS', 'PHE',
)

def SetDefaultLib(compound_lib):
  '''
  Set the default compound library. The compound library is used by various
  functions of the framework that requires knowledge of naming and 
  connectivity of residues.
  '''
  conop_inst=Conopology.Instance()
  conop_inst.SetDefaultLib(compound_lib)

def GetDefaultLib():
  conop_inst=Conopology.Instance()
  return conop_inst.GetDefaultLib()




e=io.LoadPDB("../demos/data/sdh.pdb")
scene.Add(gfx.Entity("x",gfx.HSC,e.Select("chain=A")))
scene["x"].GetOptions(gfx.HSC).SetTubeRadius(0.6)
# spiral helices (default)
scene["x"].GetOptions(gfx.HSC).SetHelixMode(0)
# straight strands
scene["x"].GetOptions(gfx.HSC).SetStrandMode(1)
# colors
scene["x"].SetColor(gfx.Color(0.9,0.95,1.0))
scene["x"].SetColor(gfx.Color(1.0,0.9,0.7),"rtype=helix")
scene["x"].SetDetailColor(gfx.Color(1.0,0.95,0.78),"rtype=helix")
scene["x"].SetColor(gfx.Color(0.5,1.0,0.7),"rtype=strand")
scene["x"].SetDetailColor(gfx.Color(0.7,1.0,0.79),"rtype=strand")

# switch to hemilighting mode
scene.SetShadingMode("hf")
# add outline to improved visibility
scene["x"].Outline(True)
scene["x"].SetOutlineMode(1)
scene["x"].SetOutlineWidth(4.5)

# looks best on white bg
scene.bg=gfx.WHITE

scene.SetRTC(geom.Mat4(-0.589889,-0.594876,0.546034,28.2245,
                        0.806165,-0.472502,0.356147,20.1685,
                        0.0461388,0.65028,0.758291,73.5665,
                        -3.52709,1.72253,-120,1))





from ost import mol
from PyQt5 import QtCore, QtWidgets, QtGui

class QueryEditorWidget(QtWidgets.QWidget):
  def __init__(self, parent=None): 
    QtWidgets.QWidget.__init__(self, parent)
    self.default_font_=QtGui.QTextCharFormat()
    self.default_font_.setForeground(QtGui.QBrush(QtGui.QColor(0,0,0)))
    self.error_font_=QtGui.QTextCharFormat()
    self.error_font_.setForeground(QtGui.QBrush(QtGui.QColor(255,0,0))) 
    self.selection_edit_=QtWidgets.QTextEdit(self)
    self.selection_edit_.setFixedHeight(40)
    self.selection_edit_.updateGeometry()
    self.status_=QtWidgets.QLabel(" ",self);
    self.status_.setWordWrap(True)
    self.status_.setMargin(0)
    self.status_.setAlignment(QtCore.Qt.AlignRight)
    self.status_.setSizePolicy(QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum,
                                                 QtWidgets.QSizePolicy.Expanding))
    vl=QtWidgets.QVBoxLayout()
    vl.addWidget(self.selection_edit_)
    self.no_bonds_=QtWidgets.QRadioButton('none')
    self.ex_bonds_=QtWidgets.QRadioButton('exclusive')
    self.in_bonds_=QtWidgets.QRadioButton('inclusive')
    self.in_bonds_.setChecked(True)
    self.match_res_=QtWidgets.QCheckBox('match residues')
    
    vl.setContentsMargins(0,0,0,0)
    vl.setSpacing(0)
    self.setLayout(vl)
    vl.addWidget(self.status_)
    hl=QtWidgets.QHBoxLayout()
    l=QtWidgets.QLabel("bonds:")

    hl.addWidget(l)
    hl.addSpacing(5)
    hl.addWidget(self.no_bonds_)
    hl.addWidget(self.ex_bonds_)
    hl.addWidget(self.in_bonds_)
    hl.addStretch(1)
    vl.addLayout(hl)
    vl.addWidget(self.match_res_)

    self.changing_text_=False;
    self.selection_edit_.textChanged.connect(self._StartTimer)
    self.timer_=QtCore.QTimer()
    self.timer_.timeout.connect(self._UpdateMessage)
    
  def GetQueryFlags(self):
    flags=0
    if self.no_bonds_.isChecked():
      flags|=mol.NO_BONDS
    if self.ex_bonds_.isChecked():
      flags|=mol.EXCLUSIVE_BONDS
    if self.match_res_.isChecked():
      flags|=mol.MATCH_RESIDUES
    return flags
  
  def GetQuery(self):
    return mol.Query(str(self.selection_edit_.toPlainText()))

  def GetQueryText(self):
    return str(self.selection_edit_.toPlainText())
  
  def SetQueryFlags(self,flags):
    self.no_bonds_.setChecked(flags & mol.NO_BONDS)
    self.ex_bonds_.setChecked(flags & mol.EXCLUSIVE_BONDS)
    self.in_bonds_.setChecked(not (flags & mol.NO_BONDS|mol.EXCLUSIVE_BONDS))
    self.match_res_.setChecked(flags & mol.MATCH_RESIDUES)
  
  def SetQuery(self,query):
    self.selection_edit_.setText(query)
    
  def _StartTimer(self):
    self.timer_.stop()
    self.timer_.start(500)

  def _UpdateMessage(self):
    if self.changing_text_:
      return
    self.changing_text_ = True
    query=self.GetQuery()
    
    cursor=self.selection_edit_.textCursor()
    cursor.select(QtGui.QTextCursor.Document)
    cursor.setCharFormat(self.default_font_)  
    
    if query.IsValid():
      self.status_.setText("")
    else:      
      d=query.GetErrorDescription()
      self.status_.setText("<font color='red'>%s</font>"%d.msg.strip())
      #self.status_.setFixedSize(self.width(),self.status_.height())
      cursor.movePosition(QtGui.QTextCursor.Start)
      if d.range.Loc<len(query.string):
        
        cursor.movePosition(QtGui.QTextCursor.NextCharacter, 
                            QtGui.QTextCursor.MoveAnchor, d.range.Loc)
        cursor.movePosition(QtGui.QTextCursor.NextCharacter, 
                            QtGui.QTextCursor.KeepAnchor, d.range.Length)
        cursor.setCharFormat(self.error_font_)
    self.changing_text_=False
    
class QueryDialog(QtWidgets.QDialog):
  def __init__(self, title, parent=None):
    QtWidgets.QDialog.__init__(self, parent)
    l=QtWidgets.QVBoxLayout(self)
    self.setWindowTitle(title)
    self.editor=QueryEditorWidget(self)
    l.addWidget(self.editor)
    l.addSpacing(10)
    l3=QtWidgets.QHBoxLayout()
    ab=QtWidgets.QPushButton('OK')
    ab.setDefault(True)
    cb=QtWidgets.QPushButton('Cancel')
    l3.addStretch(1)
    l3.addWidget(cb, 0)
    l3.addWidget(ab, 0)
    l.addLayout(l3)
    cb.clicked.connect(self.reject)
    ab.clicked.connect(self.accept)
    self.editor.selection_edit_.textChanged.connect(self._CheckNewline)

  @property 
  def query_flags(self):
    return self.editor.GetQueryFlags()
  @property
  def query(self):
    return self.editor.GetQueryText()
  
  def event(self, e):
    if e.type() == QtCore.QEvent.KeyPress and (e.key () == QtCore.Qt.Key_Return or e.key () == QtCore.Qt.Key_Enter):
      self.accept()
      return True
    else:
      return QtWidgets.QDialog.event(self, e)
  
  def _CheckNewline(self):
    if self.editor.GetQueryText().endswith("\n"):
      self.accept()



from ost.bindings import utils
from ost import settings, io, seq, LogError
import os
import subprocess

def ClustalW(seq1, seq2=None, clustalw=None, keep_files=False, nopgap=False, 
             clustalw_option_string=False):
  '''
  Runs a ClustalW multiple sequence alignment. The results are returned as a
  :class:`~ost.seq.AlignmentHandle` instance.
  
  There are two ways to use this function:
  
   - align exactly two sequences:
   
      :param seq1: sequence_one
      :type seq1: :class:`~ost.seq.SequenceHandle` or :class:`str`
      
      :param seq2: sequence_two
      :type seq2: :class:`~ost.seq.SequenceHandle` or :class:`str`
  
      The two sequences can be specified as two separate function parameters 
      (`seq1`, `seq2`). The type of both parameters can be either
      :class:`~ost.seq.SequenceHandle` or :class:`str`, but must be the same for
      both parameters.
      
   - align two or more sequences:
   
      :param seq1: sequence_list
      :type seq1: :class:`~ost.seq.SequenceList`
      
      :param seq2: must be :class:`None`
      
      Two or more sequences can be specified by using a
      :class:`~ost.seq.SequenceList`. It is then passed as the first function 
      parameter (`seq1`). The second parameter (`seq2`) must be :class:`None`.
      
       
  :param clustalw: path to ClustalW executable (used in :func:`~ost.settings.Locate`)
  :type clustalw: :class:`str`
  :param nopgap: turn residue-specific gaps off
  :type nopgap: :class:`bool`
  :param clustalw_option_string: additional ClustalW flags (see http://www.clustal.org/download/clustalw_help.txt)
  :type clustalw_option_string: :class:`str`
  :param keep_files: do not delete temporary files
  :type keep_files: :class:`bool`

  .. note ::
   
    - In the passed sequences ClustalW will convert lowercase to uppercase, and
      change all '.' to '-'. OST will convert and '?' to 'X' before aligning
      sequences with ClustalW.
    - If a :attr:`sequence name <ost.seq.SequenceHandle.name>` contains spaces,
      only the part before the space is considered as sequence name. To avoid
      surprises, you should remove spaces from the sequence name.
    - Sequence names must be unique (:class:`ValueError` exception raised
      otherwise).

  ClustalW will accept only IUB/IUPAC amino acid and nucleic acid codes:

  ======= ======================= ======= ============================ 
  Residue  Name                   Residue  Name 
  ======= ======================= ======= ============================
     A    alanine                    P    proline
     B    aspartate or asparagine    Q    glutamine
     C    cystine                    R    arginine
     D    aspartate                  S    serine
     E    glutamate                  T    threonine
     F    phenylalanine              U    selenocysteine
     G    glycine                    V    valine
     H    histidine                  W    tryptophan
     I    isoleucine                 Y    tyrosine
     K    lysine                     Z    glutamate or glutamine
     L    leucine                    X    any
     M    methionine                 \\*   translation stop
     N    asparagine                 \\-   gap of indeterminate length
  ======= ======================= ======= ============================ 

  '''
  clustalw_path=settings.Locate(('clustalw', 'clustalw2'), 
                                explicit_file_name=clustalw)
  
  if seq2!=None:
    if isinstance(seq1, seq.SequenceHandle) and isinstance(seq2, seq.SequenceHandle):
      seq_list=seq.CreateSequenceList()
      seq_list.AddSequence(seq1)
      seq_list.AddSequence(seq2)
    elif isinstance(seq1, str) and isinstance(seq2, str):
      seqh1=seq.CreateSequence("seq1", seq1)
      seqh2=seq.CreateSequence("seq2", seq2)
      seq_list=seq.CreateSequenceList()
      seq_list.AddSequence(seqh1)
      seq_list.AddSequence(seqh2)
    else:
      LogError("WARNING: Specify at least two Sequences")
      return
  elif isinstance(seq1, seq.SequenceList):
    seq_list=seq1
  else:
    LogError("WARNING: Specify either two SequenceHandles or one SequenceList")
    return

  sequence_names = set()
  for s in seq_list:
    # we cut out anything after a space to be consistent with ClustalW behaviour
    sequence_names.add(s.GetName().split(' ')[0])
  if len(sequence_names) < len(seq_list):
    raise ValueError("ClustalW can only process sequences with unique identifiers!")


  new_list = seq.CreateSequenceList()
  for s in seq_list:
    ss = s.Copy()
    for i,c in enumerate(ss):
      if c=='?':
        ss[i]='X'
    new_list.AddSequence(ss)

  seq_list = new_list

  
  temp_dir=utils.TempDirWithFiles((seq_list,))
  out=os.path.join(temp_dir.dirname, 'out.fasta')
  command='%s -infile="%s" -output=fasta -outfile="%s"' % (clustalw_path,
                                                           temp_dir.files[0],
                                                           out)
  if nopgap:
    command+=" -nopgap"
  if clustalw_option_string!=False:
    command=command+" "+clustalw_option_string  #see useful flags: http://toolkit.tuebingen.mpg.de/clustalw/help_params

  subprocess.run(command, shell=True, stdout=subprocess.DEVNULL) 

  aln=io.LoadAlignment(out)
  

  for sequence in seq_list:
    for seq_num,aln_seq in enumerate(aln.sequences):
      if aln_seq.GetName()==sequence.GetName():
        break
    aln.SetSequenceOffset(seq_num,sequence.offset)
    if sequence.HasAttachedView():
      aln.AttachView(seq_num,sequence.GetAttachedView().Copy())

  if not keep_files:
    temp_dir.Cleanup()
  
  return aln



# trajectory generated by Raimund Dutzler
from PyQt5 import QtCore
scene.RemoveAll()


class Anim(QtCore.QTimer):
  """
  Timer used to animate the trajectory on screen. Each time the OnTimer
  method gets called we advance the trajectory by one step and render
  the scene on the screen.
  """
  def __init__(self,cg,go):
      QtCore.QTimer.__init__(self)
      self.cg_=cg
      self.go_=go
      self.frame_=0
      self.timeout.connect(self.OnTimer)
      
  def OnTimer(self):
    self.frame_=(self.frame_+1)%self.cg_.GetFrameCount()
    self.go_.BlurSnapshot()
    self.cg_.CopyFrame(self.frame_)
    self.go_.UpdatePositions()
        

# load CHARMM trajectory from sample.pdb and sample.dcd
cg = io.LoadCHARMMTraj("data/sample.pdb","data/sample.dcd")
# create graphical representation of the entity
eh=cg.GetEntity()
# we don't want to display the hydrogen atoms, so select everything that is 
# not a hydrogen and pass that view to the gfx.Entity() constructor.
ev=eh.Select("not ele=H")
go=gfx.Entity("mol",gfx.SIMPLE, ev)
# enable the blur effect
go.SetBlur(True)

# add it to the scene for rendering
scene.Add(go)
scene.SetCenter(go.GetCenter())
scene.AutoAutoslab(True)

# create an animation timer and start it
anim=Anim(cg,go)
print('Demo 6: Import of a CHARMM trajectory. Type anim.stop() to halt animation, anim.start(100) to start it again with stepsize 100!Starting animation now....')
anim.start(50)



import unittest
from ost import *
from ost import settings
from ost import testutils
from ost.seq.alg import SequenceIdentity
from ost.bindings import ialign

class TestIAlign(unittest.TestCase):
  
  def setUp(self):
    self.protein = io.LoadEntity("testfiles/testprotein.pdb")


  def testIAlign(self):

    try:
      ialign_exec = settings.Locate('ialign.pl')  
    except:
      print("Could not find ialign master perl script: ignoring unit tests")
      return

    ialign_result = ialign.iAlign(self.protein, self.protein)

    # model and reference are the same, we expect pretty good results
    self.assertEqual(ialign_result.rmsd, 0.0)
    self.assertEqual(ialign_result.is_score, 1.0)
    self.assertEqual(SequenceIdentity(ialign_result.alignment), 100.0)
    self.assertEqual(ialign_result.aligned_residues, 78)
    self.assertEqual(ialign_result.aligned_contacts, 91)

    # transformation should be identity matrix (no transformation at all...)
    identity = geom.Mat4(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1)
    self.assertEqual(ialign_result.transform, identity)


if __name__ == "__main__":
  testutils.RunTests()



#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
import ost.gui.trajectory_viewer
from ._ost_gui import *

## \brief Opens a DataViewer 
# \sa \example fft_li.py "View Fourier Transform Example" \sa \ref modulate_image.py "Modulate Image Example"
def _close_event_override_(event):
  print("close event")
def _set_data_override_(data):
  print("set data")

def CreateDataViewer(ih,flag=False):
      viewer=GostyApp.Instance().CreateDataViewer(ih)
      if flag:
        viewer.image_=ih
        sip_viewer=viewer.qobject
        sip_viewer.closeEvent=_close_event_override_
        sip_viewer.setData=_set_data_override_
      return viewer

def ClearMessageWidget():
      gosty=GostyApp.Instance()
      gosty.message_widget.Clear()


from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from ost import gfx

def PickColor(default=gfx.WHITE):
  """
  Pops up a color chooser that lets' the user pick a color and returns the 
  selected color. If the user cancels the color chooser, None is returned.
  
  :rtype: :class:`~ost.gfx.Color`
  """
  dialog=QColorDialog()
  qt_color=QColor(int(min(255,default.Red()*256)),
                  int(min(255,default.Green()*256)),
                  int(min(255,default.Blue()*256)))
  qt_color=dialog.getColor(qt_color)
  if not qt_color.isValid():
    return None
  return gfx.RGBb(qt_color.red(), qt_color.green(),qt_color.blue())
                   
def GetMenu(menu_name, create=False):
  persp=GostyApp.Instance().perspective
  if isinstance(menu_name[0], QAction):
    return menu_name[0]
  else:
    node=persp.GetMenu(menu_name[0])
    for name in menu_name[1:]:
      found=False
      for action in node.actions():
        if str(action.text())==str(name):
          found=True
          node=action
          break
      if not found:
        if create:
          node=node.addMenu(name)  
        else:
          raise ValueError("Menu '%s' doesn't exist" % ', '.join(menu_name))
    return node



def AddMenuAction(*args, **kwargs):
  """
  Add menu action to main menu.

  This function lets you conveniently add actions to the main menu. To add a new 
  new action "Background Color" to the "Scene" menu, simply use
  
  .. code-block:: python
  
    def SetBackgroundColor():
      scene.bg=gfx.PickColor(scene.bg)

    AddMenuAction('Scene', "Background Color", SetBackgroundColor)
    
  This will add the menu "Scene" if it does not exist yet, register the action 
  "Background Color" and execute the function SetBackgroundColor whenever the 
  action is triggered.
  
  To assign a keyboard shortcut to the action, you can use the shortcut 
  argument:
  
  .. code-block:: python
  
    AddMenuAction('Scene', 'Background Color', SetBackgroundColor,
                  shortcut='Ctrl+B')

  Whenever you press Ctrl+B (Cmd+B on macOS), the action will be executed.
  
  Very often menu actions are coupled to the current selected objects in the 
  scene menu. These menu actions are either enabled or disabled depending on the 
  type of the selected objects. To easily support this scenario, the "enable" 
  state of the menu action can be tightly coupled to the scene menu by providing 
  a callable to the enabled argument. As an example, the following menu action 
  is only enabled when exactly one gfx.Entity is selected.
  
  .. code-block:: python
  
    AddMenuAction('Scene', 'PrintAtomCount', PrintAtomCount,
                  enabled=OneOf(gfx.Entity))
  
  OneOf is a simple function object that takes any number of classes and returns 
  true when the selected object is an instance. Alterantively, you might want to 
  use ManyOf or supply a custom function that evaluates the state of the menu 
  action to suit your needs.
  """
  class MenuActionEnabler(QObject):
    def __init__(self, predicate, action):
      QObject.__init__(self, action)
      self.predicate=predicate
      self.action=action
      app=GostyApp.Instance()
      app.scene_win.qobject.ActiveNodesChanged.connect(self.TestEnable)
      self.TestEnable()

    def TestEnable(self):
      self.action.setEnabled(self.predicate())
  persp=GostyApp.Instance().perspective
  menu_name=args[:-1]
  function=args[-1]
  if isinstance(menu_name[0], QMenu):
    node=menu_name[0]
  else:
    node=persp.GetMenu(args[0])
    for name in menu_name[1:-1]:
      found=False
      for action in node.actions():
        if str(action.text())==str(name):
          node=action
          break
      if not found:
        node=node.addMenu(name)
  action=node.addAction(str(menu_name[-1]))
  if 'shortcut' in kwargs:
    action.setShortcut(QKeySequence(kwargs['shortcut']))
  if 'checkable' in kwargs:
    action.setCheckable(kwargs['checkable'])
  if 'checked' in kwargs:
    action.setChecked(kwargs['checked'])
  if 'enabled' in kwargs:
    if callable(kwargs['enabled']):
      enabler=MenuActionEnabler(kwargs['enabled'], action)
    else:
      action.setEnabled(kwargs['enabled'])
  action.triggered.connect(function)
  return action


class OneOf:
  def __init__(self, *classes):
    self.classes=classes
  def __call__(self):
    sel=SceneSelection.Instance()
    if sel.GetActiveNodeCount()!=1:
      return False
    node=sel.GetActiveNode(0)
    for cl in self.classes:
      if isinstance(node, cl):
        return True
    return False

class TwoOf:
  def __init__(self, *classes):
    self.classes=classes
  def __call__(self):
    sel=SceneSelection.Instance()
    act_count=sel.GetActiveNodeCount()
    if act_count<2:
      return False
    found=0
    for i in range(0, act_count):
      node=sel.GetActiveNode(i)
      for cl in self.classes:
        if isinstance(node, cl):
          found += 1
      if found > 2:
        return False
    if found == 2:
      return True
    return False

class ManyOf:
  def __init__(self, *classes):
    self.classes=classes
  def __call__(self):
    sel=SceneSelection.Instance()
    if sel.GetActiveNodeCount()==0:
      return False
    for  i in range(sel.GetActiveNodeCount()):
      node=sel.GetActiveNode(i)
      found=False
      for cl in self.classes:
        if isinstance(node, cl):
          found=True
          break
      if not found:
        return False
    return True

from ost import PushVerbosityLevel as _PushVerbosityLevel
from ost import PopVerbosityLevel as _PopVerbosityLevel
from ost import GetVerbosityLevel as _GetVerbosityLevel


def PushVerbosityLevel(value):
  GostyApp.Instance().perspective.ChangeVerbositySlider(value)

def PopVerbosityLevel():
  _PopVerbosityLevel()
  GostyApp.Instance().perspective.ChangeVerbositySlider(_GetVerbosityLevel())
  _PopVerbosityLevel() # the slider change pushes a new level :-(



# run with ost_cl
struct=io.LoadEntity("sdh.pdb")
surf=io.LoadSurface("sdh","msms")
surf.Attach(struct,5.0)
go=gfx.Surface("surf",surf)
scene.Add(go)
scene.SetCenter(go.GetCenter())
scene.Autoslab()
scene.SetFog(True)
scene.Autoslab(True)
scene.fog_near_offset=10
scene.SetBackground(gfx.WHITE)

# fast version, via attached atoms
go.ColorBy("abfac",gfx.Color(0,0,1),gfx.Color(1,0,0),30,80)

scene.SetRTC(geom.Mat4(0.656168,-0.753826,-0.0341944,29.688,0.471087,0.444621,-0.761822,19.9405,0.589488,0.483781,0.646863,73.0075,-1.51607,3.36172,-100,1))

scene.ExportPov("test")




import sys
from math import *
from ost.img.alg import *
from ost.geom import *


def CreateSplitImage(imagelist, start_at_y=True):
    result=imagelist[0].Copy(False)
    result.CenterSpatialOrigin()
    extent=imagelist[0].GetExtent()
    if start_at_y:
        startpoint=Vec2(0.0,-extent.GetSize()[0])
    else:
        startpoint=Vec2(extent.GetSize()[0],0.0)
    angle=2*pi/len(imagelist)
    count=0
    for image in imagelist:
        image.CenterSpatialOrigin()
        start_angle=angle*count
        end_angle=angle*(count+1)
        pol=Polygon2()
        pol.AddNode(Vec2(0,0))
        pol.AddNode(Rotate(startpoint,start_angle))
        if(start_angle<pi/4.0 and end_angle>pi/4.0):
            pol.AddNode(Rotate(startpoint,pi/4.0))
        if(start_angle<3.0*pi/4.0 and end_angle>3.0*pi/4.0):
            pol.AddNode(Rotate(startpoint,3.0*pi/4.0))
        if(start_angle<5.0*pi/4.0 and end_angle>5.0*pi/4.0):
            pol.AddNode(Rotate(startpoint,5.0*pi/4.0))
        if(start_angle<7.0*pi/4.0 and end_angle>7.0*pi/4.0):
            pol.AddNode(Rotate(startpoint,7.0*pi/4.0))
        pol.AddNode(Rotate(startpoint,end_angle))
        m=img.Mask(pol)
        result+=image.Apply(img.alg.MaskImage(m))
        count+=1
    return result

if len(sys.argv)<3:
  imagelist=io.LoadImageList(['circle.png', 'square.png'])
else:
  imagelist=io.LoadImageList(sys.argv[1:-1])

result=CreateSplitImage(imagelist)
v_result=gui.CreateDataViewer(result,"Split Image")



# This code example shows how to calculate and display an oriented bounding box 
# that contains all the atoms of an entity.

# remove all objects from scene, just in case
scene.RemoveAll()

# Load sdh molecule
sdh=io.LoadPDB('data/sdh.pdb')
# Select backbone atoms of residues 99-128, which form a helix.
helix=sdh.Select('rnum=99:128 and cname=A and aname=CA,C,N,O')
# create a graphical representation of the helix and render it with simple 
# lines.
go=gfx.Entity('helix', gfx.SIMPLE, helix)
# add it to the scene
scene.Add(go)

# calculate the bounding box for the helix
bbox=mol.BoundingBoxFromEntity(helix)

# create a graphical representation of the bounding box and add it to the scene
bb=gfx.Cuboid('xxx', bbox)
bb.SetFillColor(gfx.Color(0.5, 0.8, 0.5, 0.2))
scene.Add(bb)

# center the camera on the graphical object
scene.center=go.center



import unittest
if __name__== '__main__':
  import sys
  sys.path.insert(0,"../../../../stage/lib64/openstructure/")
  sys.path.insert(0,"../../../../stage/lib/openstructure/")
  
import ost
from ost import geom, mol

if ost.WITH_NUMPY:
  has_numpy=True
  try:
    import numpy
  except ImportError as e:
    has_numpy=False
else:
  has_numpy=False

def v2v(v):
  return geom.Vec3(float(v[0]),float(v[1]),float(v[2]))

def dd(v1,v2):
  return geom.Distance(v1,v2)<1e-8

class TestNumpy(unittest.TestCase):
  def setUp(self):
    pass

  def test_(self):
    if not has_numpy:
      return
    entity=mol.CreateEntity()
    ed=entity.EditXCS()
    ch=ed.InsertChain("X")
    re=ed.AppendResidue(ch,"ALA")
    a0=ed.InsertAtom(re,"A",geom.Vec3(0,0,0))
    self.assertEqual(a0.GetIndex(),0)
    a1=ed.InsertAtom(re,"B",geom.Vec3(1,0,0))
    self.assertEqual(a1.GetIndex(),1)
    a2=ed.InsertAtom(re,"C",geom.Vec3(2,0,0))
    self.assertEqual(a2.GetIndex(),2)
    a3=ed.InsertAtom(re,"D",geom.Vec3(3,0,0))
    self.assertEqual(a3.GetIndex(),3)

    self.assertTrue(dd(a0.pos,geom.Vec3(0,0,0)))
    self.assertTrue(dd(a1.pos,geom.Vec3(1,0,0)))
    self.assertTrue(dd(a2.pos,geom.Vec3(2,0,0)))
    self.assertTrue(dd(a3.pos,geom.Vec3(3,0,0)))

    ed.SetAtomTransformedPos(entity.GetAtomList(),
                             numpy.array([[0,1,0],[0,2,0],[0,3,0],[0,4,0]], dtype=numpy.float32))

    self.assertTrue(dd(a0.pos,geom.Vec3(0,1,0)))
    self.assertTrue(dd(a1.pos,geom.Vec3(0,2,0)))
    self.assertTrue(dd(a2.pos,geom.Vec3(0,3,0)))
    self.assertTrue(dd(a3.pos,geom.Vec3(0,4,0)))

    na=entity.positions

    self.assertTrue(dd(v2v(na[0]),geom.Vec3(0,1,0)))
    self.assertTrue(dd(v2v(na[1]),geom.Vec3(0,2,0)))
    self.assertTrue(dd(v2v(na[2]),geom.Vec3(0,3,0)))
    self.assertTrue(dd(v2v(na[3]),geom.Vec3(0,4,0)))

    ed.SetAtomTransformedPos([3,99,2],
                             numpy.array([[0,0,-3],[-1,-1,-1],[0,0,-2]], dtype=numpy.float32))

    self.assertTrue(dd(a0.pos,geom.Vec3(0,1,0)))
    self.assertTrue(dd(a1.pos,geom.Vec3(0,2,0)))
    self.assertTrue(dd(a2.pos,geom.Vec3(0,0,-2)))
    self.assertTrue(dd(a3.pos,geom.Vec3(0,0,-3)))

if __name__== '__main__':
    unittest.main()




#------------------------------------------------------------------------------
# This file is part of the OpenStructure project <www.openstructure.org>
#
# Copyright (C) 2008-2020 by the OpenStructure authors
#
# This library is free software; you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation; either version 3.0 of the License, or (at your option)
# any later version.
# This library is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#------------------------------------------------------------------------------
# -*- coding: utf-8 -*-

from ost import gui
from ost import gfx
from ost import info
from PyQt5 import QtCore
from .immutable_gradient_info_handler import ImmutableGradientInfoHandler

#Gradient Info Handler
class GradientInfoHandler(ImmutableGradientInfoHandler):
  def __init__(self, fileName):    
    ImmutableGradientInfoHandler.__init__(self, fileName)
  
  def StoreQGradient(self,gradient,name):
    group = self.gradients_.CreateGroup(self.GRADIENT_GROUP_NAME)
    group.SetAttribute(self.NAME_ATTRIBUTE_NAME, name)
    gfx_gradient = self.ConvertToGfxGradient(gradient)
    
    gfx_gradient.GradientToInfo(group)
    self.handle_.Export(self.FILE_NAME)
    
  def RemoveGradient(self, name):
    group_list = self.gradients_.GetGroups(self.GRADIENT_GROUP_NAME)
    for group in group_list:
      if group.HasAttribute(self.NAME_ATTRIBUTE_NAME):
        groupname = group.GetAttribute(self.NAME_ATTRIBUTE_NAME)
        if name == groupname:
          groupToDel = group
          break
    
    if groupToDel != None:
      self.gradients_.Remove(groupToDel)      
      self.handle_.Export(self.FILE_NAME)
      
  def RenameGradient(self, old, new):
    group_list = self.gradients_.GetGroups(self.GRADIENT_GROUP_NAME)
    for group in group_list:
      if group.HasAttribute(self.NAME_ATTRIBUTE_NAME):
        groupname = group.GetAttribute(self.NAME_ATTRIBUTE_NAME)
        if old == groupname:
          group.SetAttribute(self.NAME_ATTRIBUTE_NAME, new)
          self.handle_.Export(self.FILE_NAME)



import unittest
from ost import *
from ost import settings
from ost import seq
from ost.bindings.clustalw import *

class TestSemiGlobalAlign(unittest.TestCase):
  def testSemiGlobalAlignment(self):
    seq_a=seq.CreateSequence('A', 'abcdefghijklmnok')
    seq_b=seq.CreateSequence('B', 'cdehijk')
    alns=seq.alg.SemiGlobalAlign(seq_a, seq_b, seq.alg.BLOSUM62)
    self.assertEqual(len(alns), 1)
    self.assertEqual(alns[0].sequences[0].name, 'A')
    self.assertEqual(alns[0].sequences[1].name, 'B')
    self.assertEqual(str(alns[0].sequences[0]), 'abcdefghijklmnok')
    self.assertEqual(str(alns[0].sequences[1]), '--cde--hijk-----')
    self.assertEqual(alns[0].sequences[0].offset, 0)
    self.assertEqual(alns[0].sequences[1].offset, 0)


if __name__ == "__main__":
  from ost import testutils
  testutils.RunTests()
